'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.splitErrorToDiagnostic = exports.errorToDiagnostic = exports.getDiagnosticSeverity = void 0;
const vscode_languageserver_1 = require("vscode-languageserver");
function getDiagnosticSeverity(severity) {
    switch (severity) {
        case 'error':
            return vscode_languageserver_1.DiagnosticSeverity.Error;
        case 'warning':
            return vscode_languageserver_1.DiagnosticSeverity.Warning;
        case 'info':
            return vscode_languageserver_1.DiagnosticSeverity.Information;
        default:
            return vscode_languageserver_1.DiagnosticSeverity.Error;
    }
}
exports.getDiagnosticSeverity = getDiagnosticSeverity;
function errorToDiagnostic(error) {
    if (error.sourceLocation.file !== undefined && error.sourceLocation.file !== null) {
        let fileName = error.sourceLocation.file;
        const errorSplit = error.formattedMessage.substr(error.formattedMessage.indexOf(fileName)).split(':');
        let index = 1;
        // a full path in windows includes a : for the drive
        if (process.platform === 'win32') {
            index = 2;
        }
        return splitErrorToDiagnostic(error, errorSplit, index, fileName);
    }
    else {
        const errorSplit = error.formattedMessage.split(':');
        let fileName = errorSplit[0];
        let index = 1;
        // a full path in windows includes a : for the drive
        if (process.platform === 'win32') {
            fileName = errorSplit[0] + ':' + errorSplit[1];
            index = 2;
        }
        return splitErrorToDiagnostic(error, errorSplit, index, fileName);
    }
}
exports.errorToDiagnostic = errorToDiagnostic;
function splitErrorToDiagnostic(error, errorSplit, index, fileName) {
    const severity = getDiagnosticSeverity(error.severity);
    const errorMessage = error.message;
    // tslint:disable-next-line:radix
    const line = parseInt(errorSplit[index]);
    // tslint:disable-next-line:radix
    const column = parseInt(errorSplit[index + 1]);
    return {
        diagnostic: {
            message: errorMessage,
            range: {
                end: {
                    character: column + error.sourceLocation.end - error.sourceLocation.start - 1,
                    line: line - 1,
                },
                start: {
                    character: column - 1,
                    line: line - 1,
                },
            },
            severity: severity,
        },
        fileName: fileName,
    };
}
exports.splitErrorToDiagnostic = splitErrorToDiagnostic;
//# sourceMappingURL=solErrorsToDiagnostics.js.map